unit uTestCase;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeEngine, CurvFitt, Series, ExtCtrls, TeeProcs, Chart, TeeEdit;

type
  TLifeTemp = class(TForm)
    chLifeTemp: TChart;
    LabData: TPointSeries;
    CurveFit: TFastLineSeries;
    TTeeFunction1: TCurveFittingFunction;
    ChartEditor1: TChartEditor;
    SmoothFit: TFastLineSeries;
    serUpper: TPointSeries;
    serlower: TPointSeries;
    procedure FormCreate(Sender: TObject);
    procedure chLifeTempDblClick(Sender: TObject);
    procedure chLifeTempAfterDraw(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  LifeTemp: TLifeTemp;

implementation

uses math;

{$R *.dfm}


procedure PlotSmoothFit( var serFit, serSmoothFit : tFastLineSeries;
                         FitFctn : TCurveFittingFunction;
                         smoothness : integer );

var interval, x, y : double; i, imax : integer; coeffs : array of double;

begin
  with FitFctn do begin
    imax := polydegree;
    SetLength( coeffs, imax );
    coeffs[ 0 ] := 0;
    for i := 2 to imax do
      coeffs[ i-1 ] := AnswerVector[ i ];
    { end }
  end;
  with serFit do begin
    interval := ( maxXvalue - minXvalue )/ Smoothness;
    for i := 1 to Smoothness do begin
      x := minXvalue + (i-1) * interval;
      y := poly( x-minXvalue, coeffs ) + minYvalue;
      serSmoothFit.AddXY( x, y )
    end;
  end
end;



procedure TLifeTemp.FormCreate(Sender: TObject);

var i : integer;

begin
  tTeeFunction1.PolyDegree := 2;
  with LabData do begin
    AddXY( 1.808, log10( 85 ));
    AddXY( 1.876, log10( 130 ));
    AddXY( 1.949, log10( 280 ));
    AddXY( 2.070, log10( 840 ));
    for i := 0 to count-1 do begin
      serUpper.AddXY( LabData.xValue[ i ], 1.02*LabData.yValue[ i ]);
      serLower.AddXY( LabData.xValue[ i ], 0.98*LabData.yValue[ i ]);
    end;
  end;
  tTeeFunction1.AddPoints( LabData );
  PlotSmoothFit( CurveFit, SmoothFit, tTeeFunction1, 500 );
end;

procedure TLifeTemp.chLifeTempDblClick(Sender: TObject);
begin
  ChartEditor1.Execute
end;

function GetNextTic( value : double ) : double;

var y0 : integer; dy, mantissa : double;

begin
  Mantissa := frac( value );  // Only for numbers > 0
  If Mantissa < 0 then mantissa := 1 + mantissa;
  y0 := round( Power( 10, mantissa ));
  case y0 of
    1,5 : dy := log10( 2 );
    2 : dy := log10( 2.5 );
  end;
  Result := dy;
end;

function GetRoundLog( input : double; upward : boolean ) : double;

var x, x1, x2, log2, log5, base : double;

begin
  log2 := log10( 2 );
  log5 := log10( 5 );
  if input = 0 then begin
    if upward then x := log2
    else x := -log2;
    result := x;
    exit;
  end;
  x := frac( input );
  if input < 0 then begin  //input = -1.82  = log (.015)
    x2 := 1+x;             // x2 = .18
    base := input - x2;    // base = -2
    x := x2
  end
  else begin
    base := input - x;
  end;
  if upward then begin
    if x < log10(2) then
      x1 := log2
    else   { >= 2 }
    if x < log10(5) then
      x1 := log5
    else
      x1 := 1
  end
  else begin {downward}
    if x > log10(5) then
      x1 := log5
  else
    if x > log10(2) then
      x1 := log2
  else
    x1 := 0
  end;
  if input < 0 then
    // x := base + (1-x1)
    x := x1 + base
  else
    x := x1 + base;
  result := x;
end;

procedure TLifeTemp.chLifeTempAfterDraw(Sender: TObject);

var
  dy, y, yMax, yMin, slope, Ea : double;  x1, y1, dy1, TI : integer;
  tpi : longint;

begin
  if not chLifeTemp.BottomAxis.inverted then
    exit;
  // How to move title away from axis?
  with LabData do begin
    yMin := MinYvalue;
    yMax := MaxYvalue;
    yMin := GetRoundLog( MinYValue, false );
    yMax := GetRoundLog( MaxYValue, true );
    // Possible mismatch between data ymin and axis ymin. work on this
    chLifeTemp.LeftAxis.SetMinMax( yMin, yMax );
    with chLifeTemp, LeftAxis, canvas do begin
      // PosTitle := PosAxis - 30; does not compile
      y := yMin;
      tpi := round( Power( 10, y ));
      // canvas.Font.Size := 7;
      Font.Size := 7;
      Pen.Style := psDot;
      // pen.Width := 1;
      // pen.color := clGray;
      TitleSize := 0;
      LabelsSize := 50;
      repeat
        y1 := CalcPosValue( y );
        DrawAxisLabel( PosAxis - 4, y1, 0, IntToStr( tpi ));
        MoveTo( PosAxis, y1 );
        LineTo( BottomAxis.IEndPos, y1 );
        dy := GetNextTic( y );
        y := y + dy;
        tpi := round( Power( 10, y ));
      until y >= yMax;
      x1 := Round( BottomAxis.iEndPos * 0.7 );
      y1 := Round( LeftAxis.iEndPos * 0.3 );
      // canvas.Font.Name := 'verdana';
      canvas.Font.Size := 9;
      dy1 := TextHeight( 'S' );
      Slope := tTeeFunction1.AnswerVector[ 2 ];
      TextOut( x1, y1, 'Slope: ' +
        FloatToStrf( slope, ffGeneral, 3, 4 ) +  'K' );
      Ea := slope * 8.31 * 2.302;  { kj/mol }
      TextOut( x1, y1 + dy1, 'Ea: ' +
        FloatToStrf( Ea, ffGeneral, 3, 4 ) + ' kj/mol' );
      {
      dy := log10( 2.0E+04 ) - maxYval;
      x := MaxXval + dy/slope;
      TI := round ( x/1000 - 273 );
      Optionally expand scale and show extrapolation with dashed line
      }
      TextOut( x1, y1 + 2*dy1, 'TI: (TBD) C at 20,000 h.' );
    end;
  end;
end;

end.
